/*
 * VeloTailLight.c
 *
 * "Автоматический велофонарь"
 *
 * Author: Погребняк Дмитрий, г. Самара, 2014
 *
 * Помещённый здесь код является свободным для некоммерческого использования, при условии указания ссылки на автора (Погребняк Дмитрий, http://aterlux.ru/).
 */ 

#define F_CPU 1000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "VeloTailLight.h"
#include "LIS3DH.h"
#include "eeprom.h"



typedef struct {
  int16_t x, y, z;
} Acc_Val;

uint8_t volatile timer_countdown;

Acc_Val acc, filter_fast;//filter_slow;
uint16_t vibration_cur, vibration_filtered;//, vibration_max;
uint8_t mode;
uint8_t current_state;
uint8_t blinker_counter;
Acc_Val home;


inline uint16_t absdiff(int16_t a, int16_t b) {
  return (a < b) ? b - a : a - b;
}

inline int16_t sfilter(int16_t oldval, int16_t newval, uint8_t shrkoef) {
  return oldval + (((int32_t)newval - oldval + (1 << (shrkoef - 1))) >> shrkoef);
}

inline uint16_t ufilter(uint16_t oldval, uint16_t newval, uint8_t shrkoef) {
  return oldval + (((int32_t)newval - oldval + (1 << (shrkoef - 1))) >> shrkoef);
}

ISR(TIMER2_OVF_vect) {
  uint8_t cd = timer_countdown;
  if (cd) {
    timer_countdown = cd - 1;
  }
}

EMPTY_INTERRUPT(BUTTON_PC_vect);

EMPTY_INTERRUPT(PCINT1_vect);

void out_num(uint8_t n) {
  LIGHT_RIGHT_PWM = (n & 1) ? 32 : 0;
  LIGHT_MIDRIGHT_PWM = (n & 2) ? 32 : 0;
  LIGHT_MIDDLE_PWM = (n & 4) ? 32 : 0;
  LIGHT_MIDLEFT_PWM = (n & 8) ? 32 : 0;
  LIGHT_LEFT_PWM = (n & 16) ? 32 : 0;
}

void port_num(uint8_t n) {
  if (n & 1) LIGHT_RIGHT_PORT |= LIGHT_RIGHT; else LIGHT_RIGHT_PORT &= ~LIGHT_RIGHT; 
  if (n & 2) LIGHT_MIDRIGHT_PORT |= LIGHT_MIDRIGHT; else LIGHT_MIDRIGHT_PORT &= ~LIGHT_MIDRIGHT; 
  if (n & 4) LIGHT_MIDDLE_PORT |= LIGHT_MIDDLE; else LIGHT_MIDDLE_PORT &= ~LIGHT_MIDDLE; 
  if (n & 8) LIGHT_MIDLEFT_PORT |= LIGHT_MIDLEFT; else LIGHT_MIDLEFT_PORT &= ~LIGHT_MIDLEFT; 
  if (n & 16) LIGHT_LEFT_PORT |= LIGHT_LEFT; else LIGHT_LEFT_PORT &= ~LIGHT_LEFT; 
}

uint8_t spi_send_recv(uint8_t data) {
  while (!( UCSR0A & (1<<UDRE0)));  
  UDR0 = data;
  while (!(UCSR0A & (1<<RXC0)));
  return UDR0;
}

uint8_t acc_read8(uint8_t reg) {
  ACC_CS_PORT &= ~ACC_CS;
  spi_send_recv(0x80 | reg);
  uint8_t res = spi_send_recv(0xFF);
  ACC_CS_PORT |= ACC_CS;
  return res;
}

int16_t acc_read16s(uint8_t reg) {
  ACC_CS_PORT &= ~ACC_CS;
  spi_send_recv(0xC0 | reg);
  int16_t res = spi_send_recv(0xFF) | (spi_send_recv(0xFF) << 8);
  ACC_CS_PORT |= ACC_CS;
  return res;
}

void acc_write8(uint8_t reg, uint8_t val) {
  ACC_CS_PORT &= ~ACC_CS;
  spi_send_recv(reg);
  spi_send_recv(val);
  ACC_CS_PORT |= ACC_CS;
}

uint8_t acc_read_xyz_new(uint8_t with_filter) {
  ACC_CS_PORT &= ~ACC_CS;
  spi_send_recv(0xC0 | STATUS_REG2);
  uint8_t have_new_data = spi_send_recv(0xFF) & (1 << ZYXDA);
  if (have_new_data) {
    acc.x = spi_send_recv(0xFF) | (spi_send_recv(0xFF) << 8);
    acc.y = spi_send_recv(0xFF) | (spi_send_recv(0xFF) << 8);
    acc.z = spi_send_recv(0xFF) | (spi_send_recv(0xFF) << 8);
    
    if (with_filter) {
      filter_fast.x = sfilter(filter_fast.x, acc.x, FILTER_FAST_SHR);
      filter_fast.y = sfilter(filter_fast.y, acc.y, FILTER_FAST_SHR);
      filter_fast.z = sfilter(filter_fast.z, acc.z, FILTER_FAST_SHR);

  /*    filter_slow.x = sfilter(filter_slow.x, acc.x, FILTER_SLOW_SHR);
      filter_slow.y = sfilter(filter_slow.y, acc.y, FILTER_SLOW_SHR);
      filter_slow.z = sfilter(filter_slow.z, acc.z, FILTER_SLOW_SHR);*/
    
      uint16_t vx = absdiff(acc.x, filter_fast.x);
      uint16_t vy = absdiff(acc.y, filter_fast.y);
      uint16_t vz = absdiff(acc.z, filter_fast.z);
      if (vy > vx)
        vx = vz;
      if (vz > vx)
        vx = vz;
      vibration_cur = vx;
  /*    vibration_max = ((uint32_t)vibration_max * FADE_VIBRATION_MAX) >> 8;
      if (vibration_cur > vibration_max)
        vibration_max = vibration_cur;*/
      vibration_filtered = ufilter(vibration_filtered, vibration_cur, FILTER_VIBRATION_SHR);
    }    
  }  
  ACC_CS_PORT |= ACC_CS;
  return have_new_data;
}

void select_odr(uint8_t ctrl_reg1) {
  acc_write8(CTRL_REG1, 0b00000000); 
  acc_write8(CTRL_REG5, 0b10000000); // BOOT
  _delay_ms(10);
  acc_write8(CTRL_REG4, 0b10001000); // high resolution, block-update mode
  acc_write8(CTRL_REG1, ctrl_reg1);
}


void power_down(uint8_t flash_light) {
  cli();
  TCCR0A = 0;
  TCCR0B = 0;
  TCCR1A = 0;
  TCCR1B = 0;
  TCCR2A = 0;
  TCCR2B = 0;
  TIMSK2 = 0;
  select_odr(0b00000000); // Полное отключение (~0,4мкА)
  
  
  LIGHT_LEFT_PORT &= ~LIGHT_LEFT;
  LIGHT_RIGHT_PORT &= ~LIGHT_RIGHT;
  if (flash_light) {
    LIGHT_MIDLEFT_PORT |= LIGHT_MIDLEFT;
    LIGHT_MIDDLE_PORT |= LIGHT_MIDDLE;
    LIGHT_MIDRIGHT_PORT |= LIGHT_MIDRIGHT;
  } else {
    LIGHT_MIDLEFT_PORT &= ~LIGHT_MIDLEFT;
    LIGHT_MIDDLE_PORT &= ~LIGHT_MIDDLE;
    LIGHT_MIDRIGHT_PORT &= ~LIGHT_MIDRIGHT;
  }
  BUTTON_PCMSK |= BUTTON_PCMSK_MASK;
  PCICR |= BUTTON_PCICR_MASK;
  set_sleep_mode(SLEEP_MODE_IDLE);
  // Ждём, пока не отпустят кнопку
  for(;;) {
    uint8_t c = 255; // С учётом дребезга, проверяем кнопку много раз подряд
    while (c && (BUTTON_PIN & BUTTON))
      c--;
    if (!c)
      break;
    sei();
    sleep_cpu();
    cli();
  }
  // Готовимся к глубокому сну
  UCSR0B &= ~((1 << RXEN0) | (1 << TXEN0));

  DIDR0 = 0b00111111;
  DIDR1 = (1 << AIN1D) | (1 << AIN0D);
  PRR = (1 << PRTWI) | (1 << PRTIM2) | (1 << PRTIM0) | (1 << PRTIM1) | (1 << PRSPI) | (1 << PRUSART0) | (1 << PRADC); // отключаем вообще всё!

  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  for(;;) {
    uint8_t c = 255; // Ждём пока кнопка не будет уверенно нажата
    while (c && !(BUTTON_PIN & BUTTON))
      c--;
    if (!c)
      break;
    uint8_t x = (MCUCR & ~(1 << BODSE)) | (1 << BODS);
    MCUCR = x | (1 << BODSE); // отключение BOD
    MCUCR = x;
    sei();
    sleep_cpu();
    cli();
  }  
  DIDR0 = 0;
  PRR = DEFAULT_PRR;
  
  BUTTON_PCMSK &= ~BUTTON_PCMSK_MASK;
  PCICR &= ~BUTTON_PCICR_MASK;

// ре-инициализация USART  
  UBRR0 = 0;
  UCSR0C = 0b11000011;
  UCSR0B |= (1 << RXEN0) | (1 << TXEN0);
  
} 

void parking_mode() {
  Acc_Val init_pos = filter_fast;
  cli();
  TCCR0A = 0;
  TCCR0B = 0;
  TCCR1A = 0;
  TCCR1B = 0;
  TCCR2A = 0;
  TCCR2B = 0;
  TIMSK2 = 0;

  LIGHT_LEFT_PORT &= ~LIGHT_LEFT;
  LIGHT_RIGHT_PORT &= ~LIGHT_RIGHT;
  LIGHT_MIDLEFT_PORT &= ~LIGHT_MIDLEFT;
  LIGHT_MIDDLE_PORT &= ~LIGHT_MIDDLE;
  LIGHT_MIDRIGHT_PORT &= ~LIGHT_MIDRIGHT;

  
  select_odr(0b00110111); // 10 Гц (~4мкА)
  //acc_write8()
  acc_write8(CTRL_REG2, 0b00110001);
  acc_write8(CTRL_REG6, 0b01000000);
  acc_write8(INT1_THS, 0b0000010);
  acc_write8(INT1_CFG, 0b00101010);
  acc_write8(INT1_DURATION, 0b00000000);
  acc_read8(REFERENCE);
  
  
  BUTTON_PCMSK |= BUTTON_PCMSK_MASK;
  PCICR |= BUTTON_PCICR_MASK;
  set_sleep_mode(SLEEP_MODE_IDLE);
  // Ждём, пока не отпустят кнопку
  for(;;) {
    uint8_t c = 255; // С учётом дребезга, проверяем кнопку много раз подряд
    while (c && (BUTTON_PIN & BUTTON))
      c--;
    if (!c)
      break;
    sei();
    sleep_cpu();
    cli();
  }

  
  
  // Готовимся к глубокому сну

  PCMSK1 |= (1 << PCINT12);
  PCICR |= (1 << PCIE1);

  DIDR0 = 0b00111111;
  DIDR1 = (1 << AIN1D) | (1 << AIN0D);
  PRR = (1 << PRTWI) | (1 << PRTIM0) | (1 << PRTIM1) | (1 << PRSPI)  | (1 << PRADC); // отключаем всё, кроме усарта и таймера 2
  
  CLKPR = (1 << CLKPCE); // Меняем прескалер с 1/8 (1 МГц) на 1/256 (31кГц)
  CLKPR = 8; 
  
//  TCCR2B = 0b00000011; // Прескалер таймера 1/32, нормальный режим, переполнение 3,8 раз в секунду
//  TIMSK2 |= (1 << TOIE2); // Разрешаем прерываение по таймеру
  
  set_sleep_mode(SLEEP_MODE_PWR_SAVE);
  uint8_t unpark_cnt = 0;
  for(;;) {
    if (ACC_INT_PIN & ACC_INT) {
      acc_read8(REFERENCE); // Сброс фильтра
    }
    if (acc_read_xyz_new(0)) {
      if (((absdiff(acc.x, init_pos.x) < PARKING_OFFSET) && (absdiff(acc.y, init_pos.y) < PARKING_OFFSET) && 
           (absdiff(acc.z, init_pos.z) < PARKING_OFFSET)) || (acc.x < -PARKING_WHEELIE_X)) {
        if (unpark_cnt)
          unpark_cnt--;
      } else {
        unpark_cnt++;
        if (unpark_cnt >= 3)
          break;
      }
    }
    PCIFR |= ((1 << PCIF0) | (1 << PCIF2) | (1 << PCIF1));
    if (!(BUTTON_PIN & BUTTON)) {
      uint8_t c = 255; // Ждём пока кнопка не будет уверенно нажата
      while (c && !(BUTTON_PIN & BUTTON))
        c--;
      if (!c)
        break;
    }      
    uint8_t x = (MCUCR & ~(1 << BODSE)) | (1 << BODS);
    MCUCR = x | (1 << BODSE); // отключение BOD
    MCUCR = x;
    sei();
    sleep_cpu();
    cli();
  } 
  
  CLKPR = (1 << CLKPCE); // Восстанавливаем прескалер 1/8 (1 МГц) 
  CLKPR = 3; 
  
  DIDR0 = 0;
  PRR = DEFAULT_PRR;
  
  BUTTON_PCMSK &= ~BUTTON_PCMSK_MASK;
  PCICR &= ~BUTTON_PCICR_MASK;
  PCMSK1 &= ~(1 << PCINT12);
  PCICR &= ~(1 << PCIE1);

  acc_write8(INT1_CFG, 0);
  acc_write8(CTRL_REG6, 0);
}

void init_lights() {
  cli();
  LIGHT_LEFT_PORT &= ~LIGHT_LEFT;
  LIGHT_MIDLEFT_PORT &= ~LIGHT_MIDLEFT;
  LIGHT_MIDDLE_PORT &= ~LIGHT_MIDDLE;
  LIGHT_MIDRIGHT_PORT &= ~LIGHT_MIDRIGHT;
  LIGHT_RIGHT_PORT &= ~LIGHT_RIGHT;
  
  GTCCR = (1 << TSM) | (1 << PSRASY) | (1 << PSRSYNC);
  OCR0A = 0;
  OCR0B = 0;
  OCR1A = 0;
  OCR1B = 0;
  OCR2A = 0;
  OCR2B = 0;
  
  TCCR0A = 0b10100001;
  TCCR0B = 0b00000001;
  TCNT0 = 0; 
  
  TCCR1A = 0b10100001;
  TCCR1B = 0b00000001;
  TCNT1 = 0; 
  
  TCCR2A = 0b10000001;
  TCCR2B = 0b00000001;
  TCNT2 = 0; 
  
  TIMSK2 |= (1 << TOIE2);
  
  GTCCR &= ~(1 << TSM);
  
  select_odr(0b01100111); // Нормальное энергопотребление, 200Гц (~38мкА), включены все 3 оси 
  sei();
}

void delay_ticks(uint8_t ticks) {
  timer_countdown = ticks;
  set_sleep_mode(SLEEP_MODE_IDLE);
  while (timer_countdown)
    sleep_cpu();
}

void set_lights(uint8_t left, uint8_t midleft, uint8_t middle, uint8_t midright, uint8_t right) {
  LIGHT_LEFT_PWM = left;
  LIGHT_MIDLEFT_PWM = midleft;
  LIGHT_MIDDLE_PWM = middle;
  LIGHT_MIDRIGHT_PWM = midright;
  LIGHT_RIGHT_PWM = right;
}

void light_and_delay(uint8_t outer, uint8_t mid, uint8_t inter, uint8_t ticks) {
  LIGHT_LEFT_PWM = LIGHT_RIGHT_PWM = outer;
  LIGHT_MIDLEFT_PWM = LIGHT_MIDRIGHT_PWM = mid;
  LIGHT_MIDDLE_PWM = inter;
  delay_ticks(ticks);
}

void calibrate_parking() {
  light_and_delay(64, 0, 0, 196);
  light_and_delay(64, 64, 0, 196);
  light_and_delay(0, 0, 128, 196);
  light_and_delay(0, 64, 0, 196);
  light_and_delay(64, 0, 0, 196);
  set_lights(0, 0, 0, 0, 0);
  uint8_t c;
  do {
    c = (BUTTON_PIN & BUTTON) ? c - 1 : 255;
  } while (c);
  uint16_t countdown = 256;
  int32_t sum_x = 0, sum_y = 0, sum_z = 0;
  Acc_Val min = {0, 0, 0}, max = {0, 0, 0};
  uint8_t button = 0;
  do {
    if (!acc_read_xyz_new(1)) {
      delay_ticks(1);
      continue;
    }
    if (filter_fast.x < min.x)
      min.x = filter_fast.x;
    else if (filter_fast.x > max.x)
      max.x = filter_fast.x;
    if (filter_fast.y < min.y)
      min.y = filter_fast.y;
    else if (filter_fast.y > max.y)
      max.y = filter_fast.y;
    if (filter_fast.z < min.z)
      min.z = filter_fast.z;
    else if (filter_fast.z > max.z)
      max.z = filter_fast.z;
    if ((vibration_filtered < STILL_VIBRATION_THRESHOLD) && ((max.x - min.x) < PARKING_OFFSET) &&
         ((max.y - min.y) < PARKING_OFFSET) && ((max.z - min.z) < PARKING_OFFSET)) {
      sum_x += acc.x;
      sum_y += acc.y;
      sum_z += acc.z;
      countdown--;
    } else {
      sum_x = sum_y = sum_z = 0;
      min.x = max.x = filter_fast.x;
      min.y = max.y = filter_fast.y;
      min.z = max.z = filter_fast.z;
      countdown = 256;
    }
    if (BUTTON_PIN & BUTTON) {
      button = 0;
    } else {
      button++;
      if (button >= BUTTON_BOUNCE_ACC) {
        set_lights(0, 0, 0, 0, 0);
        delay_ticks(255);
        return;
      }      
    }
    uint8_t outer = (countdown <= 128) ? 0 : (countdown < 192) ? countdown - 128 : 64;
    uint8_t mid = (countdown < 64) ? countdown : (countdown < 192) ? 64 : 256 - countdown;
    uint8_t inter = (countdown <= 64) ? 64 : (countdown < 128) ? 128 - countdown : 0;
    light_and_delay(outer, mid, inter, 9);
  } while (countdown);
  set_lights(0, 0, 0, 0, 0);
  home.x = sum_x >> 8;
  home.y = sum_y >> 8;
  home.z = sum_z >> 8;
  eeprom_write_int16(EEPROM_HOME_X, home.x);
  eeprom_write_int16(EEPROM_HOME_Y, home.y);
  eeprom_write_int16(EEPROM_HOME_Z, home.z);
  for (uint8_t i = 0; i < 10; i++) {
    light_and_delay(64, 0, 64, 98);
    light_and_delay(0, 64, 0, 98);
  }  
}

void switch_to_state(uint8_t newstate) {
  if (newstate == current_state)
    return;
  switch (newstate) {
    case STATE_BEACON:
      if (mode == MODE_BEACON_BLINKING) {
        set_lights(BEACON_LOW, 0, BEACON_LOW, 0, BEACON_LOW);
      } else {
        set_lights(BEACON_STILL, 0, BEACON_STILL, 0, BEACON_STILL);
      }
      break;
    case STATE_STILL:
      set_lights(STILL_LIGHT, 0, STILL_LIGHT, 0, STILL_LIGHT);
      break;
    case STATE_BRAKING:
    case STATE_EMERGENCY_BRAKING:
      set_lights(BEACON_LOW, 255, 255, 255, BEACON_LOW);
      break;
    case STATE_FALLEN:
      set_lights(255, 255, 255, 255, 255);
      break;
  }    
  blinker_counter = 0;
  current_state = newstate;
}

int main(void)
{
  DDRB = 0;
  DDRC = 0;
  DDRD = 0;
  PORTB = PORTB_UNUSED_PULLUPS;
  PORTC = PORTC_UNUSED_PULLUPS;
  PORTD = PORTD_UNUSED_PULLUPS;
  
  BUTTON_DDR &= ~BUTTON;
  BUTTON_PORT |= BUTTON;

  LIGHT_LEFT_DDR |= LIGHT_LEFT;
  LIGHT_MIDLEFT_DDR |= LIGHT_MIDLEFT;
  LIGHT_MIDDLE_DDR |= LIGHT_MIDDLE;
  LIGHT_MIDRIGHT_DDR |= LIGHT_MIDRIGHT;
  LIGHT_RIGHT_DDR |= LIGHT_RIGHT;
  
  ACSR |= (1 << ACD);
  PRR = DEFAULT_PRR;
  

  // Конфигурация портов акселерометра  
  DDRD = (DDRD | ((1 << 4) | (1 << 1))) & ~(1 << 0);
  ACC_CS_PORT |= ACC_CS;
  ACC_CS_DDR |= ACC_CS;
  
  ACC_INT_DDR &= ~ACC_INT;
  ACC_INT_PORT &= ~ACC_INT;
  
  // Конфигурация USART в режиме SPI
  UBRR0 = 0;
  UCSR0C = 0b11000011;
  UCSR0B |= (1 << RXEN0) | (1 << TXEN0);
  
  home.x = eeprom_read_int16(EEPROM_HOME_X, -1);
  home.y = eeprom_read_int16(EEPROM_HOME_Y, -1);
  home.z = eeprom_read_int16(EEPROM_HOME_Z, -1);
  sleep_enable();
  
  
//  uint8_t x = 0;
  for(;;) {
    power_down(0);
    uint8_t braking_acc = 0;
    uint8_t braking = 0;
    uint8_t emergency_braking_acc = 0;
    uint8_t emergency_braking = 0;
    uint8_t fallen_acc = 0;
    uint8_t fallen = 0;
    uint8_t still_acc = 0;
    uint8_t still = 0;
    uint8_t parking_acc = 0;
    uint8_t button_bounce = 1;
    uint8_t button_pressed = 2;
    uint16_t button_hold = 0;
    Acc_Val park_free = {0, 0, 0};
    uint16_t park_free_cnt = 0;
    init_lights();
    mode = MODE_BEACON_BLINKING;
    switch_to_state(STATE_STILL);
    for(;;) {
      uint8_t rep = 0;
      while (!acc_read_xyz_new(1)) {
        rep++;
        delay_ticks(1);
      }
      if (rep > 3) {
        select_odr(0b01100111);
      }
      switch (current_state) {
        case STATE_BEACON: {
          if (mode == MODE_BEACON_BLINKING) {
            switch (blinker_counter) {
              case 0: LIGHT_MIDDLE_PWM = BEACON_BLINK; break;
              case 32: LIGHT_MIDDLE_PWM = BEACON_LOW; break;
              case 64: LIGHT_LEFT_PWM = BEACON_BLINK; LIGHT_RIGHT_PWM = BEACON_BLINK; break;
              case 96: LIGHT_LEFT_PWM = BEACON_LOW; LIGHT_RIGHT_PWM = BEACON_LOW; break;
            }
          }        
        } break;
        case STATE_BRAKING: {
          switch (blinker_counter) {
            case 64: case 192: LIGHT_LEFT_PWM = BEACON_BLINK; LIGHT_RIGHT_PWM = BEACON_BLINK; break;
            case 96: case 224: LIGHT_LEFT_PWM = BEACON_LOW; LIGHT_RIGHT_PWM = BEACON_LOW; break;
          }
        } break;
        case STATE_EMERGENCY_BRAKING: {
          switch (blinker_counter & 63) {
            case 0: 
              LIGHT_LEFT_PWM = BEACON_BLINK; LIGHT_RIGHT_PWM = BEACON_BLINK;
            case 32: 
              LIGHT_MIDDLE_PWM = 255; LIGHT_MIDLEFT_PWM = 255; LIGHT_MIDRIGHT_PWM = 255;
              break;
            case 16: 
              LIGHT_LEFT_PWM = BEACON_LOW; LIGHT_RIGHT_PWM = BEACON_LOW;
            case 48: 
              LIGHT_MIDDLE_PWM = 0; LIGHT_MIDLEFT_PWM = 0; LIGHT_MIDRIGHT_PWM = 0;
               break;
          }
        } break;
        case STATE_FALLEN: {
          switch (blinker_counter & 63) {
            case 0: 
              LIGHT_LEFT_PWM = FALLEN_BLINK; LIGHT_MIDDLE_PWM = FALLEN_BLINK; LIGHT_MIDLEFT_PWM = FALLEN_BLINK; LIGHT_MIDRIGHT_PWM = FALLEN_BLINK; LIGHT_RIGHT_PWM = FALLEN_BLINK;
              break;
            case 32: 
              LIGHT_LEFT_PWM = FALLEN_LOW; LIGHT_MIDDLE_PWM = FALLEN_LOW; LIGHT_MIDLEFT_PWM = FALLEN_LOW; LIGHT_MIDRIGHT_PWM = FALLEN_LOW; LIGHT_RIGHT_PWM = FALLEN_LOW;
              break;
          }
        } break;
      }
      blinker_counter++;
      // Определение торможения
      if (filter_fast.x > BRAKING_ON_THRESHOLD) {
        if (braking_acc < BRAKING_CNT) {
          uint16_t add = ((uint16_t)(filter_fast.x - BRAKING_ON_THRESHOLD) >> BRAKING_ON_SPEED_RSHIFT) + BRAKING_ON_SPEED_ADD;
          if (add > BRAKING_ON_SPEED_MAX)
            add = BRAKING_ON_SPEED_MAX;
          if (add > (BRAKING_CNT - braking_acc))
            add = BRAKING_CNT - braking_acc;
          braking_acc += add;
          if (braking_acc >= BRAKING_CNT) {
            braking_acc = BRAKING_CNT;
            braking = 1;
          }          
        }      
      } else if (filter_fast.x < BRAKING_OFF_THRESHOLD) {
        if (braking_acc > 0) {
          uint16_t sub = ((uint16_t)(BRAKING_OFF_THRESHOLD - filter_fast.x) >> BRAKING_OFF_SPEED_RSHIFT) + BRAKING_OFF_SPEED_SUB;
          if (sub > BRAKING_OFF_SPEED_MAX)
            sub = BRAKING_OFF_SPEED_MAX;
          if (sub > braking_acc)
            sub = braking_acc;
          braking_acc -= sub;
          if (!braking_acc) {
            braking = 0;
          }
        }      
      }
      // Определение экстренного торможения
      if (filter_fast.x > EMERGENCY_BRAKING_THRESHOLD) {
        if (emergency_braking_acc < EMERGENCY_BRAKING_CNT_OFF) {
          emergency_braking_acc++;
          if (emergency_braking_acc >= EMERGENCY_BRAKING_CNT_ON)
            emergency_braking = 1;
        }      
      } else {
        if (emergency_braking_acc > 0) {
          emergency_braking_acc--;
          if (!emergency_braking_acc)
            emergency_braking = 0;
        }      
      }
      // Определение падения
      if ((filter_fast.y < -FALLEN_Y_THRESHOLD) || (filter_fast.y > FALLEN_Y_THRESHOLD) || (filter_fast.z < -FALLEN_Z_THRESHOLD)) {
        if (fallen_acc < FALLEN_CNT_OFF) {
          fallen_acc++;
          if (fallen_acc >= FALLEN_CNT_ON)
            fallen = 1;
        }      
      } else {
        if (fallen_acc > 0) {
          fallen_acc--;
          if (!fallen_acc)
            fallen = 0;
        }      
      }
      // Определение неподвижности
      if (vibration_filtered < STILL_VIBRATION_THRESHOLD) {
        if (still_acc < STILL_CNT_ON) {
          still_acc++;
          if (still_acc >= STILL_CNT_ON)
            still = 1;
        }      
      } else {
        if (still_acc > 0) {
          if (still_acc > STILL_CNT_OFF)
            still_acc = STILL_CNT_OFF;
          else
            still_acc--;
          if (!still_acc)
            still = 0;
        }        
      }
      // Определение парковки
      if ((still && (absdiff(filter_fast.x, home.x) < PARKING_OFFSET) && 
           (absdiff(filter_fast.y, home.y) < PARKING_OFFSET) && (absdiff(filter_fast.z, home.z) < PARKING_OFFSET)) || 
          (filter_fast.x < -PARKING_WHEELIE_X)) {
        if (parking_acc < PARKING_CNT) {
          parking_acc++;
        } else {
          parking_mode();
          parking_acc = 0;
          park_free_cnt = 0;
          init_lights();
          switch_to_state(STATE_BEACON);
          button_pressed = 2;
        }        
      } else {
        parking_acc = 0;
      }
      // Определение парковки в свободной позиции
      if (still && !fallen) {
        if (!park_free_cnt) {
          park_free.x = filter_fast.x;
          park_free.y = filter_fast.y;
          park_free.z = filter_fast.z;
          park_free_cnt = 1;
        } else if ((absdiff(park_free.x, filter_fast.x) <  PARKING_FREE_THRESHOLD) && 
           (absdiff(park_free.x, filter_fast.x) <  PARKING_FREE_THRESHOLD) && (absdiff(park_free.x, filter_fast.x) <  PARKING_FREE_THRESHOLD)) {
          if (park_free_cnt < PARKING_FREE_CNT) {
            park_free_cnt++;
          } else {          
            parking_mode();
            parking_acc = 0;
            park_free_cnt = 0;
            init_lights();
            switch_to_state(STATE_BEACON);
            button_pressed = 2;
          }          
        } else {
          park_free_cnt = 0;
        }
        
      } else {
        park_free_cnt = 0;
      }
    
      // Переход по состояниям
      switch (current_state) {
        case STATE_STILL:
          if (!still)
            switch_to_state(STATE_BEACON);
          break;
        case STATE_BEACON: 
          if (emergency_braking)
            switch_to_state(STATE_EMERGENCY_BRAKING);
          else if (braking)
            switch_to_state(STATE_BRAKING);
          else if (fallen)
            switch_to_state(STATE_FALLEN);
          else if (still)
            switch_to_state(STATE_STILL);
          break;
        case STATE_BRAKING:
          if (emergency_braking)
            switch_to_state(STATE_EMERGENCY_BRAKING);
          else if (fallen)
            switch_to_state(STATE_FALLEN);
          else if (still)
            switch_to_state(STATE_STILL);
          else if (!braking)
            switch_to_state(STATE_BEACON);
          break;
        case STATE_EMERGENCY_BRAKING:
          if (!emergency_braking) {
            switch_to_state(STATE_BRAKING);
          } else if (fallen) {
            switch_to_state(STATE_FALLEN);
          } else if (still) {
            switch_to_state(STATE_STILL);
          }
          break;
        case STATE_FALLEN:
          if (!fallen) {
            switch_to_state(STATE_BEACON);
          }
          break;
      }
  //    out_num(vibration_cur >> 7);
      //out_num(filter_slow.x >> 11);
    
      if (!(BUTTON_PIN & BUTTON)) {
        if (button_pressed == 1) {
          if (button_hold < BUTTON_HOLD_CNT) {
            button_hold++;
          } else {
            calibrate_parking();
            switch_to_state(STATE_BEACON);
            button_pressed = 2;
          }
        }
        if (button_bounce < BUTTON_BOUNCE_ACC) {
          button_bounce++;
          if ((button_pressed == 0) && (button_bounce >= BUTTON_BOUNCE_ACC)) {
            button_hold = 0;
            button_pressed = 1;
          }        
        }      
      } else {
        if (button_bounce > 0) {
          button_bounce--;
        } else { // Кнопка отпущена
          if (button_pressed == 1) {
            if (mode == MODE_BEACON_BLINKING)
              mode = MODE_BEACON_STILL;
            else if (mode == MODE_BEACON_STILL) {
              mode = MODE_FLASHLIGHT;
              break;
            }            
          }
          button_pressed = 0;
        }      
      }
  // при (dx >> 9) самый младший бит дрифтует, когда велосипед стоит на подножке
      delay_ticks(9);
    }
    power_down(1);
  }  
}