﻿/*
 * VeloTailLight.h
 *
 * "Автоматический велофонарь"
 *
 * Author: Погребняк Дмитрий, г. Самара, 2014
 *
 * Помещённый здесь код является свободным для некоммерческого использования, при условии указания ссылки на автора (Погребняк Дмитрий, http://aterlux.ru/).
 */ 


#ifndef VELOTAILLIGHT_H_
#define VELOTAILLIGHT_H_

#include <avr/io.h>
#include <avr/interrupt.h>

#define LIGHT_LEFT_PORT PORTB
#define LIGHT_LEFT_DDR DDRB
#define LIGHT_LEFT (1 << PORTB3)
#define LIGHT_LEFT_PWM OCR2A

#define LIGHT_MIDLEFT_PORT PORTB
#define LIGHT_MIDLEFT_DDR DDRB
#define LIGHT_MIDLEFT (1 << PORTB2)
#define LIGHT_MIDLEFT_PWM OCR1BL

#define LIGHT_MIDDLE_PORT PORTB
#define LIGHT_MIDDLE_DDR DDRB
#define LIGHT_MIDDLE (1 << PORTB1)
#define LIGHT_MIDDLE_PWM OCR1AL

#define LIGHT_MIDRIGHT_PORT PORTD
#define LIGHT_MIDRIGHT_DDR DDRD
#define LIGHT_MIDRIGHT (1 << PORTD6)
#define LIGHT_MIDRIGHT_PWM OCR0A

#define LIGHT_RIGHT_PORT PORTD
#define LIGHT_RIGHT_DDR DDRD
#define LIGHT_RIGHT (1 << PORTD5)
#define LIGHT_RIGHT_PWM OCR0B

#define BUTTON_PORT PORTD 
#define BUTTON_DDR DDRD
#define BUTTON_PIN PIND
#define BUTTON (1 << PIND7)
#define BUTTON_PCMSK PCMSK2
#define BUTTON_PCMSK_MASK (1 << PCINT23)
#define BUTTON_PCICR_MASK (1 << PCIE2)
#define BUTTON_PC_vect PCINT2_vect

#define ACC_CS_PORT PORTC
#define ACC_CS_DDR DDRC
#define ACC_CS (1 << PORTC5)

#define ACC_INT_PORT PORTC
#define ACC_INT_DDR DDRC
#define ACC_INT_PIN PINC
#define ACC_INT (1 << PORTC4)

#define PORTB_UNUSED_PULLUPS (0xFF & ~(LIGHT_LEFT | LIGHT_MIDLEFT | LIGHT_MIDDLE))
#define PORTC_UNUSED_PULLUPS (0xFF & ~(ACC_CS | ACC_INT))
#define PORTD_UNUSED_PULLUPS (0xFF & ~(LIGHT_MIDRIGHT | LIGHT_RIGHT | BUTTON))

#define DEFAULT_PRR ((1 << PRTWI) | (1 << PRSPI) | (1 << PRADC))

// Настройки и коэффициенты

// Коэффициенты фильтрации
#define FILTER_SLOW_SHR 4
#define FILTER_FAST_SHR 2
#define FILTER_VIBRATION_SHR 3
#define FADE_VIBRATION_MAX 252

// Состояния автомата
#define STATE_BEACON 1
#define STATE_STILL 2
#define STATE_FALLEN 3
#define STATE_BRAKING 4
#define STATE_EMERGENCY_BRAKING 5

// Режимы работы
#define MODE_OFF 0
#define MODE_BEACON_BLINKING 1
#define MODE_BEACON_STILL 2
#define MODE_FLASHLIGHT 3


// Пороги срабатывания и счётчики переходов
#define BRAKING_ON_THRESHOLD 2048
#define BRAKING_ON_SPEED_RSHIFT 9
#define BRAKING_ON_SPEED_MAX 12
#define BRAKING_ON_SPEED_ADD 1
#define BRAKING_OFF_THRESHOLD 1536
#define BRAKING_OFF_SPEED_RSHIFT 9
#define BRAKING_OFF_SPEED_MAX 16
#define BRAKING_OFF_SPEED_SUB 1
#define BRAKING_CNT 24

#define EMERGENCY_BRAKING_THRESHOLD 6144
#define EMERGENCY_BRAKING_CNT_ON 4
#define EMERGENCY_BRAKING_CNT_OFF 64

#define FALLEN_Y_THRESHOLD 8192
#define FALLEN_Z_THRESHOLD 4096
#define FALLEN_CNT_ON 250
#define FALLEN_CNT_OFF 255

#define STILL_VIBRATION_THRESHOLD 512
#define STILL_CNT_ON 250
#define STILL_CNT_OFF 4

#define PARKING_OFFSET 768
#define PARKING_CNT 250
#define PARKING_WHEELIE_X 12288
#define PARKING_FREE_THRESHOLD 512
#define PARKING_FREE_CNT 12000

// Уровни яркости
#define BEACON_LOW 48
#define BEACON_BLINK 255
#define BEACON_STILL 96
#define STILL_LIGHT 48

#define FALLEN_BLINK 255
#define FALLEN_LOW 0

// Обработка кнопки
#define BUTTON_BOUNCE_ACC 4
#define BUTTON_HOLD_CNT 400


// Адреса в EEPROM

#define EEPROM_HOME_X 0
#define EEPROM_HOME_Y 2
#define EEPROM_HOME_Z 4

#endif /* VELOTAILLIGHT_H_ */